//====================================================================================
//  Copyright (C) BAY9, 2016
//====================================================================================
//
// MODULE:
//   s2To1Ilv
//
// ----------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------

module s2To1Ilv(y, y_or, y_ff, y_we, 
                x0, x0_ir, x0_fe, x0_re, 
                x1, x1_ir, x1_fe, x1_re, 
                run1, 
                regBus, regWe, regRe, regWeOut, regReOut, clk1, clk2, reset);

  // --------------------------------------------------------------------------------------
  // Module parameters
  parameter y_w               = 0;
  parameter x0_w              = 0;
  parameter x1_w              = 0;

  parameter run1_w            = 0;

  parameter w                 = 0;

  // --------------------------------------------------------------------------------------
  // Inputs and outputs
  output            [y_w-1:0]  y;
  output                       y_or;
  input                        y_ff;
  output                       y_we;
  assign                       y_we = y_or & ~y_ff;

  input            [x0_w-1:0]  x0;
  output                       x0_ir;
  input                        x0_fe;
  output                       x0_re;
  assign                       x0_re = x0_ir & ~x0_fe;

  input            [x1_w-1:0]  x1;
  output                       x1_ir;
  input                        x1_fe;
  output                       x1_re;
  assign                       x1_re = x1_ir & ~x1_fe;

  input          [run1_w-1:0]  run1;

  // Define clock and reset
  input                        clk1;
  input                        clk2;
  input                        reset;

  // Inputs and outputs for registers
  inout                [15:0]  regBus;
  input                        regWe;
  input                        regRe;
  output                       regWeOut;
  output                       regReOut;

  // Assign clock
  wire clk;
  assign clk = clk1;

  // --------------------------------------------------------------------------------------
  // External status registers
  // --------------------------------------------------------------------------------------
  // Define register control output
  assign regWeOut = 0;
  assign regReOut = 0;

// ----------------------------------------------------------------------------------------
//=========================================================================================

  // Define registers
  reg  tglIn;

  // --------------------------------------------------------------------------------------
  // Assign input and output enable
  assign x0_ir = run1 & ~y_ff & tglIn==0;
  assign x1_ir = run1 & ~y_ff & tglIn==1;
  assign y_or  = run1 & ((tglIn==0 & ~x0_fe) | (tglIn==1 & ~x1_fe));
  assign y     = tglIn==0 ? x0 : x1;
  
  // --------------------------------------------------------------------------------------
  // FSM
  always @(posedge clk) begin
    
    // Reset
    if (run1==0) begin
      tglIn <= 0;
      
    // Normal operation
    end else begin
      
      // Register update 1
      if (y_we) begin
        tglIn <= ~tglIn;
      end

    end // if (run1_q==0) ... else
  end // always

endmodule
//=========================================================================================

